#include "about.hh"

#include "string.hh"
#include "install_dir.hh"
#include "filename.hh"
#include "vector.hh"
#include "gettext.hh"
#include "icon.hh"
#include "version_number.hh"
#include "line_reader.hh"
#include "format.hh"

#include <libgnomeui/gnome-about.h>

void About()
{
	static Vector <const char *> authors;
	if (!*authors)
	{
		String name= Install_Dir() / "AUTHORS";
		FILE *file= fopen(name(), "r");
		Verbose_Str(name());
		if (file != NULL)
		{
			Line_Reader line_reader(file);
			while (line_reader.Read())
			{
				authors.Append(strdup(line_reader.Line()));
			}
			fclose(file);
		}
		authors.Append(NULL);
		Verbose_Int(*authors);
	}

	static const char *translation;
	static bool have_translation= false;
	if (! have_translation)
	{
		have_translation= true; 
		const char *text= N_("Translator credits");
		translation= _(text);
		if (translation == text)
		{
			translation= NULL; 
		}
	}

	static GdkPixbuf *image= NULL;
	if (!image)
	{
		String name= About_Get_Image_Filename();
		Verbose_Str(name());
		image= gdk_pixbuf_new_from_file(name(), NULL);
	}	

	Verbose_Pointer(image);

	String comments= Format(_("A chess application\nfrom %s"), Version_Date());
				

	GtkWidget *about= gnome_about_new
		("BabyChess",
		 VERSION_NUMBER,
		 COPYRIGHT_LINE,
		 comments(),
		 authors(),
		 NULL,
		 translation,
		 image);
	if (!about)  return; 
	Set_Icon(GTK_WINDOW(about));
	gtk_widget_show(about); 
}

String About_Get_Image_Filename()
{
	return Install_Dir() / "share/about/title.png";
}
