#include "board_settings_editor.hh"

#include "stream_file.hh"
#include "board_settings_reset.hh"

bool Board_Settings_Editor::CE_Open(Pointer <Widget> parent,
				    const Board_Settings &settings)
{
	/* colors */ 
	if (! color_packer.Open(parent))  return false;

	assert (c_count == 6);
	static const char *const names[c_count]= 
		{ N_("_Light squares"),
		  N_("_Dark squares"),
		  N_("_Last move"),
		  N_("_Frame"),
		  N_("_Selected squares"),
		  N_("_Text color")};

	for (int i= 0;  i < c_count;  ++i)
		if (! color_choosers[i].Open(color_packer, 
					     settings.colors[i],
					     _(names[i])))
			return false;

	/* options */ 
	if (! options_packer.Open(parent))          return false;

	static const char *const text[bs_count]
		= {N_("_Turn board"),
		   N_("Hide the _frame"),
		   N_("Square _names"),
		   N_("Non-_smooth chess pieces"),
		   N_("_Tutor mode"),
		   N_("Don’t highlight the _last move"),
		   N_("_Riffled dark squares")};
	for (int i= 0;  i < bs_count;  ++i)
	{
		if (! option_boxes[i].Open(options_packer, settings.flags & (1 << i), _(text[i])))
			return false;
	}

	return true; 
}
    
int Board_Settings_Editor::EW_Use()
{
	return use_ok;
}

bool Board_Settings_Editor::OE_Retrieve(Board_Settings &object)
{
	object.flags= 0;
	for (int i= 0;  i < bs_count;  ++i)
		if (option_boxes[i].Get_Checked())
			object.flags |= (1 << i);

	for (int i= 0;  i < c_count;  ++i)
		object.colors[i]= color_choosers[i].Get_Color(); 
	return true;
}

void Board_Settings_Editor::OE_Update(const Board_Settings &settings)
{
	for (int i= 0;  i < bs_count;  ++i)
		option_boxes[i].Set_Checked(settings.flags & (1 << i)); 
	for (int i= 0;  i < c_count;  ++i)
		color_choosers[i].Set_Color(settings.colors[i]);
}
