#ifndef CONF_HH
#define CONF_HH

/* Configurations--List of saved settings. 
 */

#include "dir.hh"
#include "install_dir.hh"
#include "filename.hh"
#include "settings.hh"
#include "vector.hh"

#define CONF_SUFFIX  ".saved"
#define CONF_EXACT   ".exact"


class Conf_Dir
/* Reads the names of all configurations.  There are two types of
 * configurations:  the stock configurations (saved in PREFIX/share/babychess),
 * and the custom configurations (saved in ~/.babychess). 
 */
{
public: Conf_Dir(String new_name)
	:  name (new_name)
		{ 
			dir.Open(Stock_Dir_Name());
			stage= 0;
			stock_count= 0;
		}
	int Stock_Count() const {return stock_count;}
	String Name(int i) const
		{
			return names[i]; 
		}
	String Stock_Dir_Name() 
		{
			if (!stock_dir_name)
				stock_dir_name=  Install_Dir() / "share" / name; 
			return stock_dir_name;
		}
	String Custom_Dir_Name() 
		{
			if (! custom_dir_name)
				custom_dir_name= String(g_get_home_dir())
				     / SETTINGS_DIR_NAME
					/ name + CONF_SUFFIX;
			return custom_dir_name;
		}
	String Next();
	void Append_Custom(String name);
	void Remove_Custom(int); 

private:
	Dir dir;
	int stage;
	/* 0:  stock
	 * 1:  custom
	 */
	int stock_count;
	String name; 
	String stock_dir_name, custom_dir_name; 
	Vector <String> names;
	/* English names  */ 
};

String Conf_Exact(String conf_name);
String Conf_Exact_Filename(String conf_name);

#endif /* ! CONF_HH */ 
