#include "conf_menu.hh"

#include "settings.hh"
#include "install_dir.hh"
#include "read_line.hh"
#include "gettext.hh"
#include "stream.hh"

#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <gtk/gtkstock.h>
#include <gtk/gtktable.h>

bool Conf_Menu::Open(Pointer <Widget> parent, 
		     const char *new_type_name, 
		     Saver &new_saver)
{
	if (! Table::Open(parent, 2)) return false;
	gtk_table_set_homogeneous(GTK_TABLE(Get()), FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(Get()), dialog_unit/2);
	gtk_table_set_row_spacings(GTK_TABLE(Get()), dialog_unit/2);
	int selection= -1;

	saver= new_saver; 
	initial_exact_name= Conf_Exact(saver->Get_Type_Name()); 

	Table::fill= true; 
	if (! entry.Open(this, Entry::Open_Info
			 ("load",
			  "",
			  false, 0,
			  "edit_changed"))) 
		goto close;
	/* The entry text is set later, when we have determined the selection. 
	 */

	Table::space++;
	Table::height= 3;

	if (! list.Open(this, List::Open_Info
			(1, scr_vertical, "list", NULL, true))) 
		goto close;

	type_name= new_type_name; 

	in_list= initial_exact_name; 


	{
		const char *type_name= saver->Get_Type_Name();
		conf_dir.Alloc(type_name);
		String filename;
		int i= 0;
		while (filename= conf_dir->Next())
		{
			if (initial_exact_name == filename)
				selection= i; 
			list->Set_Row_Count(i+1);
			list->Set(0, i, _(filename()));
			++i;
		}
	}

	if (selection >= 0) list.Set_Selection(selection); 

	/* set the entry text */
	{
		String text;
		if (selection < conf_dir->Stock_Count() && selection >= 0)
		{
			const char *name= initial_exact_name();
			const char *t= _(name);
			text= t; 
		}
		else
			text= initial_exact_name; 
		entry.Set(text); 
	}

	/* widgets */ 
	Table::fill= true; 
	if (!   save_button.Open(this, "save"  , _("_Save"), true, GTK_STOCK_SAVE))  goto close; 
	Table::space++;
	Table::fill= true; 
	if (! remove_button.Open(this, "remove", _("_Remove"), true, GTK_STOCK_REMOVE))
		goto close; 
	if (selection < conf_dir->Stock_Count())
	{
		gtk_widget_set_sensitive(remove_button.Get(), false); 
		gtk_widget_set_sensitive(save_button.Get(), false);
	}
	Table::space++;
	if (! blank.Open(this))  return false; 
	
	return true; 

	close:
	Close();
	return false;
}

void Conf_Menu::MR_Receive(String message)
{
	if (message == "edit_changed")
	{
		String text= entry.Text();
		int found= -1;
		for (int i= 0;  i < list->Row_Count() && found < 0;  ++i)
			if (list->Text(0, i) == text)
				found= i;
		if (found < 0)
		{
			gtk_widget_set_sensitive(remove_button.Get(), false);
			gtk_widget_set_sensitive(save_button.Get(), true);
		}
		else 
		{
			list.Set_Selection(found);
			if (found < conf_dir->Stock_Count())
			{
				gtk_widget_set_sensitive(remove_button.Get(), false);
				gtk_widget_set_sensitive(save_button.Get(), false);
			}
			else
			{
				gtk_widget_set_sensitive(remove_button.Get(), true);
				gtk_widget_set_sensitive(save_button.Get(), true);
			}
		}
	}
	else if (message == "save")
	{
		String name= entry.Text();
		if (!name.Is())  return;
		if (! Settings_Make_Dir())  return;
		if (access(conf_dir->Custom_Dir_Name()(), F_OK))
			if (! 0 <= mkdir(conf_dir->Custom_Dir_Name()(), 0777))
				return;
		if (! saver->S_Save(conf_dir->Custom_Dir_Name() / name))  return;
		int size= list->Row_Count();
		for (int i= conf_dir->Stock_Count();  i < size;  ++i)
			if (list->Text(0, i) == name)
				return;
		list->Set_Row_Count(size+1);
		list->Set(0, size, name);
		list.Set_Selection(size); 
		gtk_widget_set_sensitive(remove_button.Get(), true);
		conf_dir->Append_Custom(name); 
	}
	else if (message == "list")
	{
		int selection= list->Selection();
		gtk_widget_set_sensitive(remove_button.Get(),
					 selection >= conf_dir->Stock_Count());
		gtk_widget_set_sensitive(save_button.Get(), 
					 selection >= conf_dir->Stock_Count());
		if (selection < 0)  return;
		String name= list->Text(0, selection);
		entry.Set(name); 
		if (! saver->S_Load
		    ((selection < conf_dir->Stock_Count() 
		      ? conf_dir->Stock_Dir_Name() :
		      conf_dir->Custom_Dir_Name()) / 
		     (selection < conf_dir->Stock_Count()
		      ? conf_dir->Name(selection)
		      : list->Text(0, selection))))  
			return;
		in_list= true;
	}

	else if (message == "remove")
	{
		int selection= list->Selection();
		if (selection < conf_dir->Stock_Count())  return;
		String name= list->Text(0, selection);
		list().Remove(selection);
		gtk_widget_set_sensitive(remove_button.Get(),
					 list->Selection() >= conf_dir->Stock_Count());
		gtk_widget_set_sensitive(save_button.Get(), 
					 list->Selection() >= conf_dir->Stock_Count());
		String filename= conf_dir->Custom_Dir_Name() / name; 
		unlink(filename()); 
		in_list= false;
		entry.Set("");
		conf_dir->Remove_Custom(selection);
	}
	else 
		Table::MR_Receive(message);
}

void Conf_Menu::Wdg_Close()
{
	saver= NULL; 
	Table::Wdg_Close(); 
}

void Conf_Menu::Save()
{
	if (in_list)
	{
		int selection= list->Selection();
		String saved= conf_dir->Name(selection);
		Save_Exact(saved);
	}
	else
	{
		String name= Conf_Exact_Filename(saver->Get_Type_Name());
		unlink(name()); 
	}
}

void Conf_Menu::Cancel()
{
	Save_Exact(initial_exact_name);
}

bool Conf_Menu::Save_Exact(String exact_name)
{
	Stream file;
	String exact_filename= Conf_Exact_Filename(saver->Get_Type_Name()); 
	if (! file.Open(exact_filename(), "w"))
		return false;

	fprintf(file(),  "%s\n",   exact_name());

	if (! file.Close())  return false;

	return true; 
}
