#ifndef CONFIGURATION_EDITOR_HH
#define CONFIGURATION_EDITOR_HH

#include "conf_menu.hh"
#include "object_editor.hh"
#include "stream_file.hh"
#include "packer.hh"

template <class Type>
class Configuration_Editor 
	:  public Object_Editor <Type> ,
	   public Conf_Menu::Saver 
{
private:
	H_Packer main_packer;
	Conf_Menu configurations; 

	virtual bool CE_Open(Pointer <Widget> parent,
			     const Type &)= 0;
	/* PARENT is an H_Packer.  */

	bool OE_Open(Pointer <Widget> parent, 
		     const Type &object)
		{
			if (! main_packer.Open(parent, Widget::dialog_unit*3/2))  
				return false;
			if (! CE_Open(main_packer, object))
				return false;
			if (! configurations.Open(main_packer, "Board_Settings", *this))  
				return false; 
			return true;
		}
	void OE_OK()
		{
			configurations.Save();
		}

	bool S_Save(String filename)
		{
			Type object;
			OE_Retrieve(object); 
			return Write_To_File(filename(), object); 
		}

	bool S_Load(String filename)
		{
			Type object; 
			if (! Read_From_File(filename(), object))
				return false;
	
			OE_Update(object); 
			return true;
		}

	void EW_Cancel() 
		{
			configurations.Cancel();
		}
};

#endif /* ! CONFIGURATION_EDITOR_HH */ 
