#include "document.hh"

#include "verbose.hh"
#include "ask.hh"

#include <fcntl.h>
#include <unistd.h>

void Document::Load()
{
	if (modified)
	{
		asked_for_what= 1;
		DO_Ask(as_load);
	}
	else
	{
		if (DO_Use_Filename())
			DO_Ask_Load_Name();
		else
		{
			DO_Save(""); 
		}
	}
}

bool Document::Save()
{
	if (Has_Filename())
		return Do_Save();
	else
	{
		Save_As();
		return false; 
	}
}

void Document::Save_As()
{
	asked_save_for= 1;
	DO_Ask_Save_Name(filename);
}

void Document::Save_Other()
{
	asked_save_for= 4;
	DO_Ask_Save_Name(filename);
}

bool Document::Do_Save()
{
	assert (filename.Is());
	if (DO_Save(filename))
	{
		modified= false;
		DO_Modified_Changed(modified);
		return true;
	}
	else
	{
		if (DO_Use_Filename())
			DO_Save_Error(); 
		return false; 
	}
}

bool Document::Asked_Load_Filename(String new_filename)
{
	assert (new_filename.Is());

	return Load_File(new_filename);
}

bool Document::Load_File(String new_filename,
			 bool anonymous)
{
	Verbose_Function(); 

	assert (new_filename.Is());

	switch (DO_Load(new_filename))
	{
	default:
		assert(0);
		return false;

	case lr_error:
		DO_Load_Error(new_filename); 
		return false;

	case lr_success:
		if ((! anonymous) && DO_Use_Filename())
		{
			filename= new_filename;
			DO_Filename_Changed(filename);
		}
		/* fall through */
	case lr_wrong_type:
		return true;
	}
}

void Document::Asked_Save_Filename(String new_filename)
{
	assert (new_filename.Is());

	filename= new_filename;
	bool success= Do_Save(); 

	if (DO_Use_Filename())
		DO_Filename_Changed(filename);

	if (! success)  return;

	switch (asked_save_for)
	{
	default:assert(0);
		break;

	case 0:	assert (DO_Use_Filename()); 
		DO_Close();
		break;

	case 1: /* Do nothing */
		break;

	case 2: DO_Ask_Load_Name(); 
		break;

	case 3: modified= false;
		DO_Modified_Changed(modified);
		DO_Reset();
		break;

	case 4: /* Do nothing */ 
		break;
	}
}

void Document::Open(String new_filename)
{
	assert (DO_Use_Filename()); 
	filename= new_filename;
	DO_Filename_Changed(filename);
}

void Document::Asked(int answer)
{
	switch (asked_for_what)
	{
	default:  assert(0);  break; 

	case 0: /* close */

		switch (answer)
		{
		default:
			assert(0);
			break;
			
		case ask_yes:
			if (DO_Use_Filename())
			{
				if (filename.Is())
				{
					if (DO_Save(filename))
					{
						DO_Close();
					}
					else
					{
						DO_Save_Error(); 
					}
				}
				else
				{
					asked_save_for= 0; 
					DO_Ask_Save_Name(filename); 
				}
			}
			else
				DO_Save_Close(); 
			break;

		case ask_no:
			DO_Close();
			break;

		case ask_cancel:
			/* Do nothing */ 
			break;
		}
		break;

	case 1: /* load */
	case 2: /* new */
		switch (answer)
		{
		default:
			assert(0);
			break;

		case ask_yes:
			asked_save_for= asked_for_what == 1 ? 2 : 3;
			DO_Ask_Save_Name(filename); 
			break;

		case ask_no:
			if (asked_for_what == 1)
				DO_Ask_Load_Name(); 
			else
			{
				modified= false;
				DO_Modified_Changed(modified);
				DO_Reset();
			}
			break;

		case ask_cancel:
			/* Do nothing */
			break;
		}
		break;
	}
}

Document:: ~Document() 
{
}

bool Document::Close_Query()
{
	if (modified)
	{
		asked_for_what= 0;
		DO_Ask(as_close);
		return false; 
	}
	else
		return true; 
}

void Document::Clear(bool ask)
{
	if ((! modified) || (! ask))
	{
		DO_Reset();
	}
	else
	{
		asked_for_what= 2;
		DO_Ask(as_clear);
	}
}

void Document::Modify()
{
	if (modified) return;
	modified= true; 
	DO_Modified_Changed(modified);
}
