#ifndef DOCUMENT_HH
#define DOCUMENT_HH

/* Implementation of the document logic:  current filename, whether is was
 * modified, query the filename, query whether to close the document...
 */

#include "string.hh"

class Document
{
public:

	/* Return value of DO_Load(). 
	 */
	enum Load_Result
	{
		lr_error= 0, 
		lr_success= 1,
		lr_wrong_type= 2,
		/* File was loaded, but is of wrong type, thus is
		 * cannot be overwritten. 
		 */

		/* ask for what */
		as_close= 0,
		as_load,
		as_clear,
	};

	Document() :  modified (false)   { }
	virtual ~Document();
	

	virtual bool DO_Use_Filename() const= 0;

	void Open(String filename);

	void Load();
	bool Save();
	/* Result:  Whether saving was done. 
	 */
	void Save_As();
	void Save_Other();
	bool Asked_Load_Filename(String filename);
	void Asked_Save_Filename(String filename);
	void Asked(int choice);
	/* Return a constant from CHOICE:  
	 * 0: no,
	 * 1: cancel
	 * 2: save/yes
	 */
	void Asked_Overwrite(int choice); 
	bool Load_File(String filename, bool anonymous= false);
	/* Load a specific file.  Return FALSE on error. 
	 */
	void Modify();
	void Unmodify()
		{
			if (! modified)  return;
			modified= false; 
			DO_Modified_Changed(modified);
		}
	bool Close_Query();
	void Clear(bool ask= true);

	bool Has_Filename() const
		{ return filename.Is();  }
	String Filename() const
		{ return filename;  }
	bool Modified() const
		{ return modified;  }

private:
	
	String filename;
	/* If empty, no filename specified.
	 */
	bool modified;
	/* Whether the document was modified. 
	 */

	int asked_for_what;
	/* What was the reason for asking.
	 *    0:   closing */
	/*    1:   loading another filename */
	/*    2:   new
	 */
	int asked_save_for;
	/* What was the reason for asking the save filename.
	 *    0:  Save before closing.
	 *    1:  Save normally. 
	 *    2:  Save before loading. 
	 *    3:  Save before new. 
	 *    4:  Save before saving as other. 
	 */

	virtual Load_Result DO_Load(String filename)= 0;
	virtual bool DO_Save(String filename)= 0;
	virtual void DO_Ask_Load_Name()= 0;
	virtual void DO_Ask_Save_Name(String initial_filename)= 0;
	/* INITIAL_NAME may be empty.  */
	virtual void DO_Filename_Changed(String)= 0;
	virtual void DO_Ask(int /* as_* */ ) {}
	/* Ask "Object was changed...save?".  Call Asked() later */ 

	virtual void DO_Load_Error(String /* filename */ ) { }
	virtual void DO_Save_Error() { }
	virtual void DO_Close() { }
		/* Close document. */
	virtual void DO_Save_Close()= 0;
	virtual void DO_Reset()= 0;
	virtual void DO_Modified_Changed(bool /* new_modified */ ) {}
	/* Called when the flags MODIFIED changes.  Can be used
	 * e.g. to display "modified" in the status bar of a document
	 * window. 
	 */

	bool Do_Save();
	/* Save to file. */

};

#endif /* ! DOCUMENT_HH */
