#include "document_editor.hh"

//#include "inform_error.hh"
#include "format.hh"
#include "gui_execute.hh"
#include "error.hh"
#include "gettext.hh"
#include "../dep/strings.hh"
#include "inform.hh"

#include <libgen.h>

void Document_Editor_Base::Close_Dialogs()
{
	ask_dialog.Close(); 
	file_dialog.Close();
}

void Document_Editor_Base::DO_Filename_Changed(String filename)
{
	if (! Widget::Is())
		return;

	if (! filename.Is())
		filename= EW_Type_Name();
	else
	{
		filename.Isolate();
		String new_name= basename(filename.c()); 
		new_name.Isolate(); 
		filename= new_name; 
	}
	Editor_Window_Base::Set_Caption(filename);
}

void Document_Editor_Base::DO_Load_Error(String filename)
{
	String message= Format(_("Cannot open \"%s\""), filename());
	if (Widget::Is())
	{
		Close_Dialogs();
		Inform_Error(message, this);
	}
	else
	{
		/* Save ERRNO because opening a window
		 * may lose ERRNO.  Also don't pass THIS as second
		 * argument, because the window is closed. 
		 */
		int e= errno;
		Inform_Error(message, NULL);
		errno= e;
	}
}
void Document_Editor_Base::DO_Ask_Load_Name()
{
	Close_Dialogs(); 
	filename_for= 1;
	file_dialog.Open(File_Dialog::t_open,
			 "asked_filename",
			 *this,
			 Document::Filename());
}

void Document_Editor_Base::DO_Ask_Save_Name(String initial_name)
{
	Close_Dialogs();
	filename_for= 2;
	file_dialog.Open(File_Dialog::t_save,  
			 "asked_filename",
			 *this,
			 initial_name);
}

void Document_Editor_Base::Append_Menu(Menubar &menubar)
{
	static GtkItemFactoryEntry owned_entries[]=
		{{N_("/File/_New"),   NULL,
		  MENU_CALLBACK, str_new,"<StockItem>",GTK_STOCK_NEW},
		 {N_("/File/_Open"),NULL,
		  MENU_CALLBACK,str_open,"<StockItem>",GTK_STOCK_OPEN},
		 {"/File/sep1",       NULL,
		  NULL,         0,            "<Separator>",0},
		 {N_("/File/_Clear"),NULL,
		  MENU_CALLBACK,str_clear,"<StockItem>",GTK_STOCK_CLEAR},
		 { N_("/File/_Save"),         NULL, 
		   MENU_CALLBACK, str_save,     "<StockItem>",  GTK_STOCK_SAVE}, 
		 {"/File/sep2",       NULL,
		  NULL,         0,            "<Separator>",0},
		 {N_("/File/_OK"),    NULL,
		  MENU_CALLBACK,str_ok,    "<StockItem>",GTK_STOCK_OK},
		 {N_("/File/_Apply"), NULL,
		  MENU_CALLBACK,str_apply, "<StockItem>",GTK_STOCK_APPLY},
		 {N_("/File/_Cancel"),NULL,
		  MENU_CALLBACK,str_cancel,"<StockItem>",GTK_STOCK_CANCEL}
		};

	static GtkItemFactoryEntry unowned_entries[]=
		{{ N_("/File/_New"),          NULL, 
		   MENU_CALLBACK, str_new,      "<StockItem>",  GTK_STOCK_NEW},  
		 { N_("/File/_Open"),         NULL, 
		   MENU_CALLBACK, str_open,     "<StockItem>",  GTK_STOCK_OPEN}, 
		 { "/File/sep1",                        NULL, 
		   NULL,          0,               "<Separator>",  0},
		 { N_("/File/_Clear"),        NULL, 
		   MENU_CALLBACK, str_clear,    "<StockItem>",  GTK_STOCK_CLEAR},
		 { N_("/File/_Load"),         NULL, 
		   MENU_CALLBACK, str_load,     NULL,           0},
		 { N_("/File/_Save"),         NULL, 
		   MENU_CALLBACK, str_save,     "<StockItem>",  GTK_STOCK_SAVE}, 
		 { N_("/File/_Save as..."),   NULL, 
		   MENU_CALLBACK, str_save_as,  "<StockItem>",  GTK_STOCK_SAVE_AS},
		 { "/File/sep2",                        NULL, 
		   NULL,          0,               "<Separator>",  0},

		 { N_("/File/_Close"),          NULL, 
		   MENU_CALLBACK, str_cancel,   "<StockItem>",  GTK_STOCK_CLOSE},
		};

	if (Get_Owner().Is())
		menubar.Append(sizeof(owned_entries)/sizeof(GtkItemFactoryEntry),
				owned_entries,
				*this);
	else
		menubar.Append(sizeof(unowned_entries)/sizeof(GtkItemFactoryEntry),
				unowned_entries,
				*this);
}

void Document_Editor_Base::MR_Receive(String message)
{
	if (message == "open")
	{
		Close_Dialogs(); 
		filename_for= 0;
		file_dialog.Open(File_Dialog::t_open,
				 "asked_filename",
				 *this,
				 Document::Filename());
	}
	else if (message == "load")
	{
		Document::Load();
	}
	else if (message == "new")
	{
		GUI_Execute(DE_Mode(), *this);
	}
	else if (message == "clear")
	{
		if (EW_Reset())
		{
			if (DO_Use_Filename())
				Document::Open("");
		}
	}
	else if (message == "asked")
	{
		int choice= ask_dialog.Get_Value(); 
		Document::Asked(choice); 
		ask_dialog.Have_Read(); 
	}
	else if (message == "asked_filename")
	{
		switch (filename_for)
		{
		case 0: GUI_Execute(DE_Mode(),
				    file_dialog.Filename(), *this);  
			return;
		case 1: Document::Asked_Load_Filename(file_dialog.Filename());
			return;
		case 2: Document::Asked_Save_Filename(file_dialog.Filename());
			return;
		}
		file_dialog.Have_Read(); 
	}
	else if (message == "save")
	{
		if (Get_Owner())
		{
			Document::Save_Other();
		}
		else
			Document::Save();
	}
	else if (message == "save_as")
	{
		Document::Save_As();
	}
	else 
		Editor_Window_Base::MR_Receive(message);
}

void Document_Editor_Base::Append_Tool_Buttons(Toolbar &toolbar)
{
	if (Get_Owner().Is())  
	{
		ok_button= toolbar.Append(GTK_STOCK_OK, "ok",  _("OK")); 
		apply_button= toolbar.Append(GTK_STOCK_APPLY, "apply",  _("Apply")); 
		toolbar.Append(GTK_STOCK_CANCEL, "cancel",  _("Cancel")); 
		toolbar.Append_Space(); 
		toolbar.Append(GTK_STOCK_CLEAR, "clear", _("Clear"));
	}
	else
	{
		toolbar.Append(GTK_STOCK_NEW, "new",  _(DE_Text_New())); 
		toolbar.Append(GTK_STOCK_OPEN, "open",  _(DE_Text_Open())); 
		toolbar.Append(GTK_STOCK_SAVE, "save", _(DE_Text_Save()));
	}
}

void Document_Editor_Base::Update_Valid(Menubar &menubar)
{
	if (Get_Owner().Is())
	{
		gtk_widget_set_sensitive(ok_button, valid);
		gtk_widget_set_sensitive(apply_button, valid);
		menubar.Set_Sensitive("/File/OK", valid);
		menubar.Set_Sensitive("/File/Apply", valid);
	}
}

void Document_Editor_Base::DO_Save_Error()
{
	String error= Error::Get();
	Close_Dialogs();
	Inform_Error(Format(_("Cannot save \"%s\""),
			    Document::Filename()()),
		     this);
}

void Document_Editor_Base::DO_Ask(int as)
{
	Close_Dialogs();

	String message;
	switch (as)
	{
	default:  assert(0);  break;
	case Document::as_close:
		if (Filename())
			message= Format(_("Save changes to document \"%s\" before closing?"), 
					Filename()());
		else
			message= _("Save the document before closing?");
		break;
	case Document::as_load:
		if (Filename())
			message= Format(_("Save changes to document \"%s\"?"), 
					Filename()());
		else
			message= _("Save the document?");
		break;
	case Document::as_clear:
		if (Filename())
			message= Format(_("Save changes to document \"%s\" before clearing?"), 
					Filename()());
		else
			message= _("Save the document before clearing?");
		break;
	}

	ask_dialog.Open(message,
			"asked",
			*this, 
			GTK_STOCK_NO,
			GTK_STOCK_CANCEL,
			GTK_STOCK_SAVE);
}

bool Document_Editor_Base::EW_Open()
{
	if (! Window::Open(Wdg_Init_Caption(), DE_Window_Size())) 
		return false;
	if (! packer.Open(this))           goto close_window;
	if (! Document_Editor_Open(packer))goto close_packer;  
		
	valid= true; 

	return true; 

 close_packer:
	packer.Close();
 close_window:
	Window::Close();
	
	return false; 
}

void Document_Editor_Base::DO_Close()
{
	this->Close(true); 
}

String Document_Editor_Base::Wdg_Init_Caption() const
{
	return this->EW_Type_Name(); 
}

bool Document_Editor_Base::Wdg_Close_Query()
{
	return Document::Close_Query();   
}

GdkPoint Document_Editor_Base::DE_Window_Size() const
{
	return (GdkPoint){unit*30, unit*20};   
}

bool Document_Editor_Base::DO_Use_Filename() const
{
	return ! this->Get_Owner().Is(); 
}

void Document_Editor_Base::DO_Save_Close() 
{
	this->MR_Send("ok"); 
}

void Document_Editor_Base::DO_Reset()  
{ 
	DE_Reset();   
}

bool Document_Editor_Base::EW_Transient() const
{ 
	return false;  
}

void Document_Editor_Base::EW_Apply()
{
	Unmodify();  
}

bool Document_Editor_Base::EW_Reset()
{
	Document::Clear(! this->Get_Owner().Is());
	return true;
}

bool Document_Editor_Base::DE_Set_Args(Open_Info &, bool &)
{
	Verbose_Function(); 
	return true; 
}
