#include "engine_menu.hh"

#include "gettext.hh"
#include "conf.hh"

bool Engine_Menu::Open(Pointer <Widget> parent, 
		       const Engine &new_engine,
		       String new_msg_change)
{
	Verbose_String(new_engine.program);

	engine= new_engine;
	msg_change= new_msg_change; 

	if (! H_Packer::Open(parent, dialog_unit/2))   return false;

	H_Packer::Set_Next(true);
	if (! menu.Open(this, "menu_change"))  goto close;
	Update_List();
	
	if (! button.Open(this, "other", _("Other...")))
		goto close;

	return true;

 close: Close();
	return false;
}

void Engine_Menu::MR_Receive(String message)
{
	if (message == "other")
	{
		engine_editor.Open("edited", 
				   *this,
				   engine,
				   this);
	}
	else if (message == "edited")
	{
		conf_dir./*Free*/reset(); 
		menu.Clear();
		Update_List();
	}

	else if (message == "menu_change")
	{
		Parent_Send_Message(msg_change); 
	}

	else
		H_Packer::MR_Receive(message);
}

void Engine_Menu::Wdg_Close()
{
	engine_editor.Close(); 
	H_Packer::Wdg_Close(); 
}

void Engine_Menu::Retrieve(Engine &new_engine)
{  
	int selection= menu.Selection();

	if (selection >= 0)
	{
	String filename= 
		(selection < conf_dir->Stock_Count() 
		 ? conf_dir->Stock_Dir_Name() 
		 : conf_dir->Custom_Dir_Name()) / 
		conf_dir->Name(selection);
	Read_From_File(filename(), engine);
	}
	
	new_engine= engine; 
}

void Engine_Menu::Update_List()
{
	String current= Conf_Exact("engine");
	int current_index= -1;
	int i= 0;
	conf_dir.reset(new Conf_Dir("engine"));   //.Alloc("engine");
	{
		String name;
		while (name= conf_dir->Next())
		{
			menu << name; 
			if (name == current)
				current_index= i;
			++i; 
		}
	}
	menu.Update();
	if (current_index >= 0) menu.Set_Selection(current_index); 
}

String Engine_Menu::Name() const
{
	String ret= conf_dir->Name(menu.Selection());
	if (menu.Selection() < conf_dir->Stock_Count())
		ret= _(ret()); 
	return ret; 
}
