#include "evaluator_settings_editor.hh"

#include "gettext.hh"

bool Evaluator_Settings_Editor::OE_Open(Pointer <Widget> parent,
					const Evaluator_Settings &settings)
{
	/* reply */ 
	if (! start_chooser.Open(parent, "", _("Start the engine"), 3))
		return false;

	start_chooser() << _("_Never")
			<< _("After each _user move")
			<< _("_Always");
	start_chooser->Set_Selection(settings.start);

	/* timing */ 
	if (! chooser.Open(parent, "timing_changed", _("Stop the engine"), 2))  
		return false;
	chooser() << _("_Never")
		  << _("After a _fixed time");
	chooser->Set_Selection(settings.timing);

	Spin_Button::Open_Info oi= {"", settings.time_s, 1, INT_MAX};
	String label= Format("%s (s)", _("_Time"));
	if (! time.Open(parent, label(), oi))  return false; 

	/* flags */ 
	if (! flags_packer.Open(parent))  return false;
	for (int i= 0;  i < evf_count;  ++i)
	{
		assert (evf_count == 4);
		static const char *const text[evf_count]=
			{N_("Run engine with high _priority"),
			 N_("_Play move when found"),
			 N_("Start engine when there is _one legal move"),
			 N_("Play moves from the _opening book")};
		if (! flags[i].Open(flags_packer,
				    settings.flags & (1 << i),
				    _(text[i])))
			return false;
	}

	return true; 
}


bool Evaluator_Settings_Editor::OE_Retrieve(Evaluator_Settings &settings)
{
	settings.timing= (Timing) chooser->Selection();
	if (settings.timing == -1)
		settings.timing= ti_infinite;
	settings.time_s= time.button.Get_Value();
	settings.start= start_chooser->Selection(); 

	settings.flags= 0;

	for (int i= 0;  i < evf_count;  ++i)
		if (flags[i].Get_Checked())
			settings.flags |= (1 << i);

	return true; 
}

void Evaluator_Settings_Editor::MR_Receive(String message)
{
	if (message == "timing_changed")
	{
		gtk_widget_set_sensitive
			(time.button.Get(), 
			 chooser->Selection() == ti_time);
	}
	else
		Object_Editor <Evaluator_Settings> ::MR_Receive(message); 
}
