#include "fen.hh"

#include "verbose.hh"
#include "format.hh"

String FEN(const Position &position)
{
	String result;

	/* First field:  piece placement */

	for (int rank= sq_8; rank >= sq_1; -- rank)
	{
		if (rank != sq_8)  result += '/';

		int empty_count= 0;

		for (int file= sq_a; file <= sq_h; ++ file)
		{
			int piece= position[Make_Square(file, rank)];
	    
			if (piece & pi_color)
			{
				if (empty_count != 0)
				{
					result += '0' + empty_count;
					empty_count= 0;
				}

				const char *const chars= "PNBRQK";
				if (piece & pi_white)
					result += chars[piece & pi_type];
				else
					result += 'a' - 'A' + chars[piece & pi_type];
			}
			else
			{
				++ empty_count;
			}
		}

		if (empty_count != 0)
			result += '0' + empty_count;
	}

	result += ' ';

	/* Second field:  active color */

	if (position.To_Move() == cc_w)
		result += 'w';
	else
		result += 'b';

	result += ' ';

	/* Third field:  castling availability */

	unsigned osize= *result;
	if (position.Moved(1) == 0 &&
	    position[sq_e1] == pi_wK)
	{
		if (position.Moved(2) == 0 &&
		    position[sq_h1] == pi_wR)
			result += 'K'; 
		if (position.Moved(0) == 0 &&
		    position[sq_a1] == pi_wR)
			result += 'Q'; 
	}
	if (position.Moved(4) == 0 &&
	    position[sq_e8] == pi_bK)
	{
		if (position.Moved(5) == 0 &&
		    position[sq_h8] == pi_bR)
			result += 'k'; 
		if (position.Moved(3) == 0 &&
		    position[sq_a8] == pi_bR)
			result += 'q'; 
	}

	if (*result == osize)
		result += '-';

	result += ' ';

	/* Fourth field:  en passant target square */

	if (position.EP_Poss() == 0)
		result += '-';
	else
	{
		result += 'a' + File(position.EP_Poss()) - sq_a;
		if (Rank(position.EP_Poss()) == sq_4)
			result += '3';
		else
			result += '6';
	}

	result += ' ';

	/* Fifth field:  halfmove clock */

	result += Format("%d", position.Halfmove_Clock()); 

	result += ' ';

	/* Sixth field:  fullmove number */

	result += Format("%d", position.previous_move_count / 2 + 1);

	return result;
}
