#include "match_window.hh"

#include "about.hh"
#include "help_menu.hh"
#include "parse_move.hh"
#include "inform.hh"
#include "ask.hh"
#include "game_stream.hh"
#include "notation_stream.hh"
#include "notation_reset.hh"
#include "../dep/strings.hh"
#include "gui_execute.hh"

bool Match_Window::Open(Open_Info)
{
	notation_preferences.Open((*this), "notation");
	if (! Window::Open(_("Play Chess"), (GdkPoint){25*unit, 17*unit}))  return false;
	if (! overall_packer.Open(this))  goto close; 


	static GtkItemFactoryEntry menu_items[] = 
		{{N_("/_Game"), NULL, NULL, 0, "<Branch>", 0},
		 {N_("/Game/_New"), NULL,
		  MENU_CALLBACK, str_new, "<StockItem>", GTK_STOCK_NEW},
		 {N_("/Game/_Save"), NULL, 
		  MENU_CALLBACK, str_save, "<StockItem>", GTK_STOCK_SAVE},
		 { "/Game/sep1",            NULL,         
		   NULL,          0,                   "<Separator>", 0},
		 { N_("/Game/Take _back last move"), NULL, MENU_CALLBACK,
		   str_take_back, NULL, 0},
		 {N_("/Game/_Resign"), NULL, MENU_CALLBACK, str_resign,
		  NULL, 0},
		 { N_("/Game/_Analyze"), NULL, MENU_CALLBACK, str_analyze,
		   NULL, 0},
		 { "/Game/sep2",            NULL,         
		   NULL,          0,                   "<Separator>", 0},
		 { N_("/Game/_Close"),       NULL, 
		   MENU_CALLBACK, str_ok,    "<StockItem>", GTK_STOCK_CLOSE},
		 { N_("/_View"),            NULL,         
		   NULL,          0,                   "<Branch>",    0},
		 { N_("/View/_Notation"),NULL,
		   MENU_CALLBACK, str_edit_notation, "<StockItem>",  GTK_STOCK_PREFERENCES},
		 BOARD_MENU
		 OPENING_WIDGET_MENU
		 HELP_MENU
		};

	if (! menubar.Open(overall_packer, 
			   sizeof (menu_items) / sizeof (GtkItemFactoryEntry),
			   menu_items,
			   *this))  goto close;
	game.New();
	playing= false;
	modified= false;

	overall_packer.Set_Next(true); 
	if (! main_packer.Open(overall_packer))  goto close;
	if (! side_packer.Open(main_packer))  goto close;
	if (! players_display.Open(side_packer))  goto close;
	if (! opening_widget.Open(side_packer, menubar, game, "opening"))
		goto close;
	side_packer.Set_Next(true); 
	if (! game_list.Open(side_packer, game, notation_preferences(),
			     "game_list"))
		goto close; 
	if (! material_widget.Open(side_packer))  goto close;
	if (! promotion_chooser.Open(side_packer)) goto close; 
	promotion_chooser.Set_Visible(false);

	if (! board.Open(main_packer,
			 Board::Open_Info
			 (game.Current_Position(),
			  "move",
			  "",
			  "turn", "", "",
			  game.Current_Generator(),
			  false,
			  menubar)))
		goto close;
	
	if (! status_bar.Open(overall_packer, 2))  goto close; 

	players_editor.Open("players_edited", *this);

	Update_Position();
	opening_widget.Update();
	Update_Playing();

	return true;

 close:
	Window::Close();
	return false;
}

void Match_Window::MR_Receive(String message)
{
	if (!memcmp(message(), "opening_", 8))
		opening_widget.MR_Receive(message() + 8);
	else if (!memcmp(message(), "board_", 6))
		board.MR_Receive(message);

	else if (message == "players_edited") /* New game */ 
	{
		if (modified)
		{
			ask_dialog.Open(_("Save the game before beginning a new game?"),
					"asked-new",
					*this, 
					GTK_STOCK_NO,
					GTK_STOCK_CANCEL,
					GTK_STOCK_SAVE);
			return;
		}

		playing= true;
		modified= false;
		Update_Playing();

		/* stop the game */
		match_players[cc_w].Free();
		match_players[cc_b].Free();

		/* set starting position */
		game.New(); 
		
		/* initialize players */
		players= players_editor.Get_Value();
		players_editor.Have_Read();
		for (int i= 0;  i < 2;  ++i)
		{
			if (players.players[i].type != p_human)
				if (! New_Match_Player(match_players[i],
						       players.players[i],
						       i ? "black":"white",
						       *this))
				{
					Inform_Error(Format(_("Cannot start %s"),
							    players.players[i].name()),
						     this);
					return;
				}
		}
		for (int i= 0;  i < 2 ; ++i)
		{
			players_display.Set_Name(i, players.players[i].name);
			game.info.players[i]= players.players[i].name; 
		}
		

		/* Start engine */ 
		if (match_players[game.Current_Position().To_Move()])
		{
			match_players[game.Current_Position().To_Move()]->MP_Go();
		}

		/* Hide widgets */
		promotion_chooser.Set_Visible(false);
		material_widget.Hide();

		/* Caption */
		String title= Format("%s – %s",
				     players.players[cc_w].name(),
				     players.players[cc_b].name());
		gtk_window_set_title
			(GTK_WINDOW(Root().Get()), title());

		/* Display the side to move */
		Update_Position(); 

		/* Clear board */
		board.Invalidate_Position();
		board.Clear_Last_Move(); 

		/* Turn board */
		if ((players.players[cc_w].type == p_human) !=
		    (players.players[cc_b].type == p_human))
			board.Set_Turned(players.players[cc_b].type == p_human); 
		
		/* Reset clocks */
		players_display.Reset_Clocks(); 

		/* Opening */
		opening_widget.Update();

		/* Game move list */
		game_list.Update();
	}
	else if (message == "new")
	{
		players_editor.Open("players_edited", *this);
	}

	else if (message == "save")
	{
		file_dialog.Open(File_Dialog::t_save,
				 "asked_name",
				 *this);
	}

	else if (message == "asked-close"  ||
		 message == "asked-new")
	{
		int what= message == "asked-close" ? 0 : 1;
		int choice= ask_dialog.Get_Value(); 
		switch (choice)
		{
		default:  assert (0);  break;
		case ask_no:
			if (what)
			{
				modified= false; 
				MR_Receive("players_edited");
			}
			else
				Close(true);
			break;
		case ask_cancel:
			break;
		case ask_yes:
			file_dialog.Open(File_Dialog::t_save,
					 what ? "asked_name-new" : "asked_name-close",
					 *this);
			break;
		}
		ask_dialog.Have_Read(); 
	}

	else if (message == "asked_name-new" ||
		 message == "asked_name-close" ||
		 message == "asked_name")
	{
		if (! Write_To_File(file_dialog.Filename()(),
				    game))
		{
			Inform_Error(Format(_("Saving \"%s\""),
					    file_dialog.Filename()()),
				     this);
		}
		else 
		{
			modified= false; 
			if (message == "asked_name-new")
				MR_Receive("players_edited");
			else if (message == "asked_name-close")
				Close(true); 
		}
		file_dialog.Have_Read(); 
	}

	/* user move */ 
	else if (message == "move")
	{
		if (! playing)  return; 

		int from, to;
		if (! board.Get_Squares(from, to))  return;

		/* it's an engine's turn:  don't make the move */ 
		if (match_players[game.Current_Position().To_Move()])
			return;

		int index= game.Current_Generator().Square_Move(from, to);
		if (index < 0)  return; /* no such move */ 

		if (game.Current_Generator()[index].promotion & pi_color)
			index += promotion_chooser.Selection();

		players_display.Set_To_Move(1^game.Current_Position().To_Move());
		board.Invalidate_Move_Squares(game.Current_Generator()[index]);
		board.Set_Last_Move(from, to);

		/* pass the move to the engine if it is now an engine’s turn */ 
		if (match_players[1^game.Current_Position().To_Move()])
		{
			match_players[1^game.Current_Position().To_Move()]->MP_Move
				(Simple_Notation(game.Current_Generator()[index]));
		}

		game.Append_Move(index);
		Check_End_Of_Game(); 
		Update_Last_Move();
		Update_Position(); 
		opening_widget.Update();
		game_list.Update_Length();
		game_list.Update_Content_One(game.Overall_Location()-1);
		game_list.Update_Highlight();
		modified= true;
	}

	else if (message == "game_list")
	{
	}

	else if (message == "resign")
	{
		if (! playing)  return;

		/* check that the user is to move */
		if (match_players[game.Current_Position().To_Move()])
			return;

		/* set score */
		game.info.result= 1<<(1^game.Current_Position().To_Move());

		/* finish game */ 
		Update_Last_Move();
		Finish_Game();
		modified= true;
	}
	else if (message == "take_back")
	{
		if (! playing)   return; 
		Go_Backward();
		int to_move= game.Current_Position().To_Move();
		if (match_players[to_move])
		{
			Go_Backward();
			match_players[to_move]->MP_Take_Back();
		}
		if (match_players[1^to_move])
			match_players[1^to_move]->MP_Take_Back();
		game.Append_Variation(); 
		game_list.Update_Length();
		game_list.Update_Content(game.Overall_Location());
		game_list.Update_Highlight();
		int l= game.Overall_Location();
		if (l)
		{
			const Move &move= game.Overall_Move(l-1);
			board.Set_Last_Move(move.from, move.to);
		}
		else
			board.Clear_Last_Move(); 
	}

	else if (message == "turn")
	{
		material_widget.Set_Turned(game.Current_Position(), board.Turned());
		players_display.Set_Turned(board.Turned()); 
	}

	else if (message == "notation")
	{
		game_list.Update();
		Update_Last_Move(); 
	}
	else if (message == "edit_notation")
	{
		notation_preferences.Edit(); 
	}
	else if (message == "analyze")
	{
		String filename= Write_To_Tmp_File(game);
		if (filename == "" || ! GUI_Execute("edit-game", filename(), "-a", *this))
		{
			Inform_Error(_("Cannot open game"), this);
			return;
		}
	}

	/* engine move */ 
	else if ((!memcmp(message(),"black ", 6))||
		 (!memcmp(message(),"white ", 6)))
	{
		int color= message[0] == 'b';
		if (color != game.Current_Position().To_Move())
		{
			/* Engines may send moves at the wrong time.*/
			Verbose("wrong time"); 
			return;
		}
		String move= message()+6;
		int move_index= Parse_Move(move, game.Current_Generator());
		if (move_index < 0)  return;
		gdk_beep(); 
		const Move &m= game.Current_Generator()[move_index];
		board.Invalidate_Move_Squares(m);
		board.Set_Last_Move(m.from, m.to);

		/* pass the move to the engine if it is now an engine’s turn */ 
		if (match_players[1^game.Current_Position().To_Move()])
		{
			match_players[1^game.Current_Position().To_Move()]->MP_Move
				(Simple_Notation(game.Current_Generator()[move_index]));
		}

		game.Append_Move(move_index);
		Check_End_Of_Game();
		Update_Last_Move();
		Update_Position(); 
		opening_widget.Update();
		game_list.Update_Length();
		game_list.Update_Content_One(game.Overall_Location()-1);
		game_list.Update_Highlight();
		players_display.Set_To_Move(game.Current_Position().To_Move());
		modified= true;
	}

	else
		Window::MR_Receive(message);
}

void Match_Window::Update_Last_Move()
{
	status_bar.Set(0, game.Format_Last_Move(notation_preferences())); 
	promotion_chooser.Update(game.Current_Position());
	menubar.Set_Sensitive("/Game/Take back last move", 
			      game.Overall_Location() != 0);
}

void Match_Window::Update_Position()
{
	status_bar.Set(1, playing ? game.To_Move_Description() : "");

	material_widget.Update(game.Current_Position(),
			       board.Turned(),
			       side_packer);
}

void Match_Window::Check_End_Of_Game()
{
	if (0 == *game.Current_Generator())
	{
		game.info.result= game.Current_Position().In_Check()
			? (1^game.Current_Position().To_Move())<<1
			: re_draw; 
		Finish_Game();
	}
}

void Match_Window::Finish_Game()
{
	playing= false;
	match_players[cc_w].Free();
	match_players[cc_b].Free(); 
	players_display.Stop(); 
	Update_Playing();
}

bool Match_Window::Wdg_Close_Query()
{
	if (modified)
	{
		ask_dialog.Open(_("Save the game before closing?"),
				"asked-close",
				*this, 
				GTK_STOCK_NO,
				GTK_STOCK_CANCEL,
				GTK_STOCK_SAVE);
		return false; 
	}
	else
		return true;
}

void Match_Window::Wdg_Close()
{
	Verbose_Function(); 

	ask_dialog.Close();
	file_dialog.Close();
	players_editor.Close(); 
	match_players[cc_w].Free();
	match_players[cc_b].Free(); 
	notation_preferences.Close(); 

	Window::Wdg_Close(); 
}

void Match_Window::Update_Playing()
{
	menubar.Set_Sensitive("/Game/Resign", playing);
	if (! playing)
		menubar.Set_Sensitive("/Game/Take back last move", false);
}

void Match_Window::Go_Backward()
{
	if (game.Overall_Location() == 0)  return; 
	Move move= game.Overall_Move(game.Overall_Location() - 1);
	while (game.Current_Location() == 0)
		game.Leave_Variation(); 
	game.Go_Backward(); 
	board.Invalidate_Move_Squares(move); 
}


