#ifndef MATCH_WINDOW_HH
#define MATCH_WINDOW_HH

#include "packer.hh"
#include "board.hh"
#include "game.hh"
#include "process_open_info.hh"
#include "match_player.hh"
#include "players_editor.hh"
#include "status_bar.hh"
#include "promotion_chooser.hh"
#include "players_display.hh"
#include "material_widget.hh"
#include "notation_editor.hh"
#include "ask_dialog.hh"
#include "opening_widget.hh"
#include "game_list.hh"
#include "file_dialog.hh"
#include "window.hh"

class Match_Window
	:  public Window
{
public:
	typedef Process_Open_Info Open_Info; 

	bool Open(Open_Info);

private:
	V_Packer overall_packer; 
	H_Packer main_packer;
	V_Packer side_packer; 
	Players_Display players_display; 
	Opening_Widget opening_widget; 
	Menubar menubar; 
	Material_Widget material_widget; 
	Promotion_Chooser promotion_chooser; 
	Board board;
	Status_Bar status_bar; 
	Game_List game_list; 

	Players players;
	Dynamic <Match_Player> match_players[2];
	/* empty when played by human.  */
	Owned_Editor <Players_Editor> players_editor; 
	Preferences <Notation, Notation_Editor> notation_preferences;
	bool playing;
	bool modified; 
	Ask_Dialog ask_dialog; 
	Game game;
	/* The game may be non-normalized when a move was taken back.
	 * Move are always appended to the current variation, which is
	 * empty after taking back a move.  Therefore, Go_Backward()
	 * must do special things:  leave the current variation if it is
	 * empty before taking back the last move.  When Book will be
	 * used instead of Game, all this will go away. 
	 */
	File_Dialog file_dialog; 

	void Wdg_Close();
	bool Wdg_Close_Query();
	
	void MR_Receive(String);

	void Update_Last_Move();
	void Update_Position();
	void Check_End_Of_Game(); 
	void Finish_Game();
	void Update_Playing(); 
	void Go_Backward();
};

#endif /* ! MATCH_WINDOW_HH */
