#include "notation_editor.hh"

#include "verbose.hh"
#include "stream_file.hh"
#include "notation_stream.hh"
#include "notation_reset.hh"
#include "gettext.hh"

bool Notation_Editor::CE_Open(Pointer <Widget> parent,
			      const Notation &notation)
{
	/* strings */ 
	static const char *const texts[s_count]=
		{N_("_En passant"), 
		 N_("_Short castling"), 
		 N_("_Long castling"), 
		 N_("_Pawn"),
		 N_("K_night"),
		 N_("_Bishop"), 
		 N_("_Rook"),
		 N_("_Queen"),
		 N_("_King"), 
		 N_("_Capture"),
		 N_("_Promotion"),
		 N_("_White"),
		 N_("_Black"),
		 N_("_Check"),
		 N_("_Double check")};
		
	if (! strings.Open(parent, s_count, texts))  return false;
	for (int i= 0;  i < s_count;  ++i)
		strings.Set(i, notation[i]);

	/* options */ 
	if (! option_packer.Open(parent))        return false;
	if (! type.Open(option_packer, "type", _("Notation type"), 1))  return false;
	type() << _("_Short algebraic")
	       << _("_Long algebraic") 
	       << _("_Descriptive");
	type->Set_Selection(Selection(notation.options));
	for (int i= 0;  i < nt_count;  ++i)
	{
		static const char *const text[nt_count]=
			{N_("Use the piece sign on _pawn moves"),
			 N_("Indicate _en passant captures"),
			 N_("Show _check and mate"),
			 N_("Use the _capture sign in short notation"),
			 N_("Capture sign at the _end in short notation"),
			 N_("Capture sign for _pawn moves in short notation"),
			 N_("Display the _captured piece"),
			 N_("Indicate _double check")};
		if (! check_box[i].Open(option_packer,
					notation.options & (1 << i << nt_base),
					_(text[i])))
			return false; 
	}

	return true; 
}

bool Notation_Editor::OE_Retrieve(Notation &notation)
{
	/* options */ 
	notation.options &= ~ ((1 << nt_used) - 1);
	for (int i= 0;  i < nt_count;  ++i)
		if (check_box[i].Get_Checked())  notation.options |= (1 << i << nt_base);
	int selection= type->Selection();
	assert (selection >= 0);
	if (selection >= 0)  notation.options |= Options(selection);

	/* strings */
	for (int i= 0;  i < s_count;  ++i)
		notation[i]= strings[i];
	return true; 
}

void Notation_Editor::OE_Update(const Notation &notation)
{
	type->Set_Selection(Selection(notation.options)); 

	for (int i= 0;  i < nt_count;  ++i)
		check_box[i].Set_Checked(notation.options & (1 << i << nt_base)); 

	for (int i= 0;  i < s_count; ++i)
		strings.Set(i, notation[i]);

	Update_Sensitive(); 
}

void Notation_Editor::MR_Receive(String message)
{
	if (message == "type")
	{
		Update_Sensitive();
	}
	else 
		Object_Editor <Notation> :: MR_Receive(message); 
}

void Notation_Editor::Update_Sensitive()
{
	bool short_sensitive= type->Selection() == 0; 
	gtk_widget_set_sensitive(check_box[3].Get(), short_sensitive);
	gtk_widget_set_sensitive(check_box[4].Get(), short_sensitive);
	gtk_widget_set_sensitive(check_box[5].Get(), short_sensitive);
}
