#ifndef OBJECT_EDITOR_HH
#define OBJECT_EDITOR_HH

/* A window that edits an object.  The window is always owned by
 * another window.    */

#include "editor_window.hh"
#include "bevel.hh"
#include "text_button.hh"
#include "dialog.hh"

template <class Type>
class Object_Editor
	:  public Editor_Window <Type> ,
	   public Dialog
{
public:
	virtual bool OE_Retrieve(Type &object)= 0; 
	/* Take the current settings and put them in OBJECT.  */
	virtual void OE_Update(const Type &) {}

	const Type &EW_Get_Value() 
		{
			if (Is())   OE_Retrieve(object);
			return object;
		}

private:
	Type object; 

	virtual bool OE_Get_Apply() const { return false;  }
	virtual bool OE_Open(Pointer <Widget> parent, const Type &)= 0; 
	virtual void OE_OK() {}
	virtual bool OE_Hide_Apply() const { return false; }
	virtual const char *OE_OK_Text() const
		{ return NULL; }

	bool EW_Open()
		{
			if (! Dialog::Dialog_Open(OE_Hide_Apply(),
						  OE_OK_Text()))     
				return false; 
			if (! OE_Open(this, object))    return false; 
			return true; 
		}
	void EW_Set_Value(Pointer <const Type> value)
		{
			if (value.Is()) object= value();
			else            object.Reset();
		}
	void EW_OK()
		{
			OE_Retrieve(object); 
			OE_OK();
		}

	GdkPoint DE_Window_Size() const { return (GdkPoint){-1, -1}; }
	void Wdg_Close()
		{
			OE_Retrieve(object);
			Editor_Window <Type> ::Wdg_Close(); 
		}
}; 

#endif /* ! OBJECT_EDITOR_HH */ 
