#include "opening_book.hh"

#include "install_dir.hh"
#include "filename.hh"
#include "opening_settings.hh"
#include "gettext.hh"
#include "format.hh"
#include "randomize.hh"

#include <ctype.h>

bool Opening_Book::Open()
{
	assert (! Is());
	String book_filename= Install_Dir() 
		/ "share/opening_book/openings.book"; 
	if (! book.Open_Read(book_filename()))  return false;
	has= false; 
	return true;
}

bool Opening_Book::Get(const Game &game)
{
	assert (Is());

	book.Go_To_Start();

	if (game.Starting_Position().Is())
		goto has_not;

	for (int i= 0; i < game.Overall_Location(); ++i)
	{
		if (! book.Go_Forward(game.Overall_Move(i)))
			goto has_not;
	}
	has= true;
	return true; 
 has_not: 
	has= false;
	return false;
}

void Opening_Book::Get(const Game &game,
		       Vector <String> &names,
		       int op_flags)
{
	assert (Is());

	book.Go_To_Start();
	if (book.Comment())
		names.Append(book.Comment());
	for (int i= 0;  i < game.Overall_Length();  ++i)
	{
		Move move= game.Overall_Move(i);
		if (! book.Go_Forward(move))  
			break; 
		String comment= book.Comment(); 
		    
		if (comment)
		{
			
			if (*names && names[-1][0] == '*')
				names= -1;
			

			int ast= 0;
			if (comment[0] == '*')
			{
				ast= true;
				String n= comment()+1;
				comment= n;
			}
			int plus_sign_count= 0;
			while (comment[plus_sign_count] == '+')
				++ plus_sign_count;
			comment= Decode(comment()+plus_sign_count, op_flags);
			if (ast)
			{
				String n= String('*')+comment; 
				comment= n;
			}

			Verbose("Removing %d plus signs", plus_sign_count); 
			names= ((*names - plus_sign_count) >? 0);
			
			names.Append(comment); 
		}
	}
	if (! (op_flags & op_subvariation))
	{
		if (*names)  names= 1;
	}
	if (*names && names[-1][0] == '*')
	{
		names[-1]= names[-1]()+1; 
	}
	Get(game);
}

bool Opening_Book::Choose_Random_Opening(move_t &small_move) const
{
	assert (Is());

	/* Count current moves */ 
	Book::usigf_t next= book().position.current;
	int count= 0, sum= 0;
	while (next)
	{
		Book::sigf_t score= book[book[next].move.after].position.score;
		if (score < 0)  score= 0;
		sum += score; 
		++count;
		next= book[next].move.cnext;
	}
	if (count == 0)  return false;

	/* Choose randomly */ 
	int ends[count], index= 0;
	next= book().position.current;
	while (next)
	{
		Book::sigf_t score= book[book[next].move.after].position.score;
		ends[index]= score;
		if (index)  ends[index] += ends[index-1];
		next= book[next].move.cnext;
		++index;
	}
	assert (index == count);
	int move= -1;
	if (ends[index-1])  
	{
		Randomize(); 
		int x= rand() % ends[index-1];
		next= book().position.current;
		for (int i= 0;  i < count; ++i)
		{
			if (x < ends[i])  
			{
				move= i;
				break;
			}
		}
	}
	else 
		move= 0;
	assert (move >= 0);

	/* Return move */
	next= book().position.current;
	while (move)
	{
		next= book[next].move.cnext;
		--move;
	}
	small_move= book[next].move.move;
	return true; 
}

String Opening_Book::Name(move_t small_move, int op_flags)
{
	assert (Is());

	if (! has)  return "";
	Book::usigf_t move= book.Get_Move(small_move);
	if (!move)  return "";
	Book::usigf_t comment=
		book[book[move].move.after].position.comment;
	if (!comment)  return _("(in book)"); 
	String ret= book.Get_String(comment);
	const char *p= ret();
	while (isspace(*p) || *p == '*')  ++p;
	String n= p;
	return Decode(n, op_flags);
}

String Opening_Book::Decode(String comment, int op_flags)
{
	Vector <String> alternatives;
	const char *p= comment();
	while (p < comment()+*comment)
	{
		const char *g= p;
		int paren_depth= 0;
		while (*g && (*g != ';' || paren_depth))  
		{
			if (*g == '(')  ++paren_depth;
			else if (*g == ')') --paren_depth;
			paren_depth >?= 0;
			++g;
		}
		if (alternatives.Set(*alternatives+1))
		{
			const char *q= g;
			while (isspace(q[-1])) --q;
			alternatives[-1]= String(p, q-p);
		}
		if (! (op_flags & op_alternative)) break;
		p= g;
		if (*p == ';')  ++p;
	}

	String ret;
	for (int i= 0;  i < *alternatives;  ++i)
	{
		if (ret)  ret += ", ";
		ret += Decode(alternatives[i]);
		if (i==0)
		{
			const char *p= ret();
			while (*p && *p != ';')  ++p;
			if (*p)
			{
				if (! (op_flags & op_alternative))
					ret= String(ret(), p-ret());
				else
					ret.c()[p-ret()]= ',';
			}
		}
	}
	return ret; 
}

String Opening_Book::Decode(String comment)
{
	//	assert (Is());

	const char *p= comment();
	while (isspace(*p))  ++p;
	const char *q= p;
	while (*q && *q != '(')   ++q;
	if (!*q)  return comment;

	const char *r= q+1;
	int depth= 1;
	while (depth)
	{
		switch (*r++)
		{
		case '\0':  return comment;
		case '(':   ++depth;  break;
		case ')':   --depth;  break;
		}
	}
	String arg(q+1, r-q-2);
	if (*p == '_')
	{
		Verbose("\ttranslating \"%s\"", arg()); 
		return _(arg());
	}
	String arg_text= Decode(arg);
	while (q > p && isspace(q[-1])) --q;
	String function(p, q-p);
	const char *text= "%s";

	static const char *functions[][2]=
		{{"V", N_("%s Variation")},
		 {"D", N_("%s Defense"  )},
		 {"A", N_("%s Attack"   )},
		 {"G", N_("%s Gambit"   )},
		 {"S", N_("%s System"   )},
		 {"F", N_("%s Formation")},
		 {"M", N_("%s Mate"     )},
		 {"L", N_("%s Line"     )},
		 {"T", N_("%s Trap"     )},
		 {"O", N_("%s Opening"  )},
		 {"E", N_("%s Move"     )},
		 {"a", N_("Anti-%s"     )},
		 {"b", N_("Classical %s"  )},
		 {"c", N_("Counter-%s"  )},
		 {"d", N_("%s Deferred" )},
		 {"e", N_("Semi-%s"       )},
		 {"f", N_("Delayed %s"    )},
		 {"g", N_("Modern %s"     )},
		 {"r", N_("%s Reversed" )},
		 {"n", N_("Neo-%s"      )},
		 {"x", N_("Accelerated %s")},
		 {"l", N_("Closed %s"     )},
		 {"o", N_("Open %s"       )},
		 {"p", N_("Pseudo-%s"     )},
		 {"m", N_("Primitive %s"  )},
		 {"v", N_("Improved %s"   )},
		 {"y", N_("Old %s"        )},
		 {"Ac", N_("%s Counterattack")},
		 {"Gc", N_("%s Countergambit")},
		};
	for (int i= 0; i < (int)(sizeof(functions)/(2*sizeof(const char *))); ++i)
		if (function == functions[i][0])
		{
			text= functions[i][1];
			break;
		}

	return Format(_(text), arg_text());
}
