#ifndef OWNED_EDITOR_HH
#define OWNED_EDITOR_HH

#include "dynamic.hh"
#include "widget.hh"

/* TYPE is the editor window type.  TYPE should be derived from
 * Editor_Window <T>.  TYPE should define Get_Value() as a way to
 * access the data in write protected mode.  Type::Edited must be
 * defined as the type that is edited.   
 */

template <class Type_Editor>
class Owned_Editor 
	:  public Dynamic <Type_Editor>
{
public:

	~Owned_Editor() { assert (! Dynamic <Type_Editor> ::Is()); }

	void Open(String msg,
		  Widget &owner)
		{
			typename Type_Editor::Edited edited;
			edited.Reset();
			Open(msg, owner, edited, &owner);
		}

	void Open(String msg,
		  Message_Receiver &owner, 
		  const typename Type_Editor::Edited &edited,
		  Pointer <Widget> parent_window,
		  String caption= "")
		{
			if (Dynamic <Type_Editor> ::Is())
			{
				if ((*this)().Is())
				{
					gtk_window_present(GTK_WINDOW((*this)().Root().Get())); 
					return;
				}
				else
				{
					Dynamic <Type_Editor> ::Free();
					Dynamic <Type_Editor> ::Alloc();
				}
			}
			else
			{
				Dynamic <Type_Editor> ::Alloc();
			}
			if (Dynamic <Type_Editor> ::Is())
			{
				(*this)().Open(msg, owner, edited, parent_window, caption);
			}
		}

	const typename Type_Editor::Edited &Get_Value()  { return (*this)().EW_Get_Value(); }

	void Close();
	/* Always force */

	bool Have_Read()
		/* Called after the command is received.  Try to close
		 * the window.  Return TRUE if window was closed.
		 * Return FALSE if the window is still open.  
		 */
		{
			if (! Dynamic <Type_Editor> ::Is())
				return true;
			if (! (*this)().Is())
			{
				Dynamic <Type_Editor> ::Free();
				return true; 
			}
			if ((*this)().Close_Editor(false))
			{
				Dynamic <Type_Editor> ::Free(); 
				return true;
			}
			else
			{
				return false; 
			}
		}

	bool Is_Editing() const { return Dynamic <Type_Editor> ::Is() && (*this)().Is();   }
	void Update_Caption()
		{
			if (Dynamic <Type_Editor> ::Is() && (*this)().Is())
			{
				(*this)().Update_Caption(); 
			}
		}
};

#if 0
template <class Type_Editor> void Owned_Editor <Type_Editor> ::
Open(String msg,
     Message_Receiver &owner, 
     const typename Type_Editor::Edited &edited,
     Pointer <Widget> parent_window,
     String caption)
{
	if (Dynamic <Type_Editor> ::Is())
	{
		if ((*this)().Is())
		{
			gtk_window_present(GTK_WINDOW((*this)().Root().Get())); 
			return;
		}
		else
		{
			Dynamic <Type_Editor> ::Free();
			Dynamic <Type_Editor> ::Alloc();
		}
	}
	else
	{
		Dynamic <Type_Editor> ::Alloc();
	}
	if (Dynamic <Type_Editor> ::Is())
	{
		(*this)().Open(msg, owner, edited, parent_window, caption);
	}
}
#endif /* 0 */ 


template <class Type_Editor>
void Owned_Editor <Type_Editor> ::Close()
{
	if (Dynamic <Type_Editor> ::Is())
	{
		if ((*this)().Is())
		{
			bool closed= (*this)().Close_Editor(true);
			assert (closed);
		}
		Dynamic <Type_Editor> ::Free();
	}
}

#endif /* ! OWNED_EDITOR_HH */
