#include "player_widget.hh"

#include <gtk/gtktable.h>

bool Player_Widget::Open(Pointer <Widget> parent,
			 const Player &player,
			 int color)

{
	if (! V_Packer::Open(parent, dialog_unit))  return false;
	if (! type_packer.Open(this, dialog_unit))    goto close;

		if (! color_packer.Open(type_packer, dialog_unit/2))  goto close;
		if (! piece.Open(color_packer, Piece(color, pi_K)))  goto close; 
		if (! color_label.Open
		    (color_packer, 
		     color ? I_("player:Black") : I_("player:White"), 
		     0.5,
		     "size=\"xx-large\""))  goto close;

	type_packer.Set_Next(true); 
	if (!type.Open(type_packer, "type-change", 2))
		goto close; 
	type.width= 2;
	blank.Open(type);
	type.space++; 
	gtk_table_set_homogeneous(GTK_TABLE(type.Get()), FALSE);
	type << _("_Human");
	type.space++;
	type << _("_Engine:"); 

	type.Set_Selection(player.type); 
	if (! engine_menu.Open(type, player.engine, "engine_change"))  goto close; 

	if (! name.Open(this, _("Player _name")))
		goto close; 
	MR_Receive("type-change"); 
	if (player.type == p_engine)
		MR_Receive("engine_change");
	else
		name.entry.Set(player.name); 
	return true;

 close:
	Close();
	return false;
}

void Player_Widget::Set(const Player &player)
{
	assert (player.type >= 0 && player.type < p_count);
	type.Set_Selection(player.type);
	engine_menu.Update(player.engine);
	MR_Receive("type-change"); 
	if (player.type == p_engine)
		MR_Receive("engine_change"); 
}

void Player_Widget::Retrieve(Player &player)
{
	player.type= type.Selection();
	if (player.type < 0)  player.type= 0;
	engine_menu.Retrieve(player.engine);
	player.name= name.entry.Text();
}

void Player_Widget::MR_Receive(String message)
{
	if (message == "type-change")
	{
		gtk_widget_set_sensitive(GTK_WIDGET(engine_menu.Get()),
					 type.Selection() == 1);
	}

	else if (message == "engine_change")
	{
		name.entry.Set(engine_menu.Name());
	}

	else  V_Packer::MR_Receive(message); 
}

