#ifndef PREFERENCES_HH
#define PREFERENCES_HH

#include "settings.hh"
#include "owned_editor.hh"
#include "stream.hh"

template <class Type, class Type_Editor>
class Preferences
	:  public Message_Receiver
{
public:
	
	enum {menu_main}; 

	~Preferences()
		{ }

	void Open(Widget &new_owner,
		  String new_msg_changed)
		{
			owner= new_owner;
			msg_changed= new_msg_changed;
			name= ((Type_Editor*)NULL)->Type_Editor::Get_Type_Name();
			bool loaded= Settings_Load(object, name);
			if (! loaded && Type::Get_Default_Configuration_Name())
			{
				object.Reset();
				if (Settings_Make_Dir())
				{
					String filename=
						Settings_Filename(String(name) + ".exact");
					Stream file;
					if (file.Open(filename(), "w"))
					{
						String setting= Type::
							Get_Default_Configuration_Name();
						fprintf(file(), "%s\n",
							setting());
						file.Close(); 
					}
				}
			}
		}

	void Close()
		{
			editor.Close(); 
			owner= NULL; 
		}

	const Type &Get_Value() const
		{
			return object; 
		}
	const Type &operator () () const
		{
			return Get_Value(); 
		}
	const Type *operator -> () const
		{
			return & Get_Value(); 
		}
	Type &Get_Change_Value() 
		/* The value can be changed, but Changed() must be
		 * called after the change. 
		 */
		{
			return object;
		}

	const Type &Get_Old_Value() const
		/* Only called inside MSG_CHANGED. 
		 */
		{
			return *old_object/*()*/;
		}
	void Changed()
		/* Must be called after changes were made to the
		 * object using Get_Change_Value().  Doesn't need to
		 * be called if not saved to file. 
		 */
		{ Settings_Save(object, name);  }
	void Edit()
		{
			editor.Open("edited", *this, object, owner()); 
		}
	void Caption_Changed()
		{
			if (editor.Is())
				editor.Update_Caption(); 
		}

private:

	Type object;
	std::auto_ptr <Type> old_object;
//	Dynamic <Type> old_object; 
	Owned_Editor <Type_Editor> editor;
	String msg_changed;
	Pointer <Widget> owner; 
	const char *name;

	void MR_Receive(String message)
		{
			if (message == "edit")
				Edit(); 

			else if (message == "edited")
			{				
				if (old_object.reset(Data_Alloc(Type, object)), !old_object.get())
//				if (! old_object.Alloc(object))
					return;
				object= editor.Get_Value();
				owner->MR_Receive(msg_changed);
				old_object/*.Free()*/.reset();
				editor.Have_Read(); 
				Settings_Save(object, name);
			}
		}	       
};

#endif /* ! PREFERENCES_HH */
