#include "scaled_board_image.hh"

#include "board_image.hh"
#include "format.hh"
#include "filename.hh"
#include "stream_file.hh"
#include "pixmap_stream.hh"
#include "image.hh"
#include "dir.hh"
#include "piece.hh"

#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

//#if defined (IMAGE_HAVE_MAP) && defined (IMAGE_HAVE_DRAW_STRETCH_CANVAS)
//#    define SCALED_BOARD_IMAGE_HAVE_MAP
//#endif

Vector <Dynamic <Scaled_Board_Image::Image_Info> > 
Scaled_Board_Image::images, Scaled_Board_Image::aa_images; 

bool Scaled_Board_Image::Load(int new_width,
			      bool new_anti_alias)
{
	assert (new_width > 0);

	/* already loaded */ 
	if (new_width == width && new_anti_alias == anti_alias)
		return true;

	/* close */ 
	if (width >= 0)  Remove();

	anti_alias= new_anti_alias;

	/* already open in list */ 
	for (int i= 0; i < *Get_Images(); ++i)
	{
		if (Get_Images()[i]().width == new_width)
		{
			++ Get_Images()[i]().ref_count;
			width= new_width;
			index= i;
			return true;
		}
	}

	/* append new entry to list */ 
	if (! Get_Images().Set(*Get_Images() + 1))    return false;

	if (! Get_Images()[-1].Alloc())
	{
		Get_Images()= -1;
		return false;
	}

	index= *Get_Images() - 1;

	Get_Images()[-1]().ref_count= 1;
	Get_Images()[-1]().width= width= new_width;

	GdkPoint dimension= {6 * new_width, 2 * new_width};
	Pointer <Alpha_Pixmap> image= Get_Images()[-1]().image; 

	String dir= String(g_get_home_dir()) / "tmp"; 
	if (access(dir(), F_OK))   mkdir(dir(), 0777);

#if 0
	/* Now load scaled images saved in temp directory, either as a
	 * map or as a file. 
	 */
//#ifdef SCALED_BOARD_IMAGE_HAVE_MAP

	/* Load map */

	String map_type= Image::Get_Map_Type(); 
	String map_name= dir /
		Format("babychess.scaled_board_image.%s%d.%s.map", 
		       anti_alias ? "aa." : "",
		       width,
		       map_type());
	String alpha_map_name= dir /
		Format("babychess.scaled_board_image.alpha.%s%d.%s.map", 
		       anti_alias ? "aa." : "",
		       width,
		       map_type()); 
	Dbg("\tReading maps...");
	Dbg("\t\t%s", map_name());
	Dbg("\t\t%s", alpha_map_name()); 

	Image image;
	Image alpha_image;
	if (image.Open_Map(map_name))
	{
		if (alpha_image.Open_Map(alpha_map_name))
		{
			Dbg("\tOpened both maps.");
			if (image.Get_Size() == dimension        &&
			    alpha_image.Get_Size() == dimension  &&
			    image().Get_Image().Create(dimension)       && 
			    image().Get_Image().Has_Canvas()            &&
			    image().Get_Alpha_Image().Create(dimension) &&
			    image().Get_Alpha_Image().Has_Canvas()      &&
			    image.Send_Rect(image().Get_Image()(),
						   dimension,
						   0)                   &&
			    alpha_image.Send_Rect(image().Get_Alpha_Image()(),
							 dimension,
							 0))
			{
				return true; 
			}


			Dbg("\tMaps have wrong size."); 
			alpha_image.Free(); 
		}
		image.Free(); 
	}
	Dbg("\tOpening maps failed."); 

//#else /* ! SCALED_BOARD_IMAGE_HAVE_MAP */ 
#endif /* 0 */ 

	/* open saved scaled images */ 

	String image_filename= dir /
		Format("babychess.scaled_board_image.%s%d" /*PIXMAP_STREAM_SUFFIX*/, 
		       anti_alias ? "aa." : "",
		       width);
	String alpha_image_filename= dir /
		Format("babychess.scaled_board_image.alpha.%s%d" /*PIXMAP_STREAM_SUFFIX*/, 
		       anti_alias ? "aa." : "",
		       width);

	if (Read_From_File(image_filename(), image().Get_Image())   &&
	    Read_From_File(alpha_image_filename(), image().Get_Alpha_Image()))
	{
		Verbose("\tLoaded from file.");
		Verbose("\timage      = (%d x %d)", 
		    image().Get_Image().Get_Size().x,
		    image().Get_Image().Get_Size().y);
		Verbose("\talpha_image= (%d x %d)", 
		    image().Get_Alpha_Image().Get_Size().x,
		    image().Get_Alpha_Image().Get_Size().y);
		if (image->Get_Image().Get_Size().x != dimension.x  ||
		    image->Get_Image().Get_Size().y != dimension.y  ||
		    image->Get_Alpha_Image().Get_Size().x != dimension.x ||
		    image->Get_Alpha_Image().Get_Size().y != dimension.y)
			/* images have wrong size */
		{
			image->Get_Image().Free();
			image->Get_Alpha_Image().Free();
			goto generate;
///			width= -1;
//			return false;
		}
		return true;
	}

	
//	if (image().Get_Image().Is())
//		image().Get_Image().Free();

//#endif /* ! SCALED_BOARD_IMAGE_HAVE_MAP */ 

	/* generate from source */ 
 generate:
	if (! (image().Get_Image().Create(dimension)       && 
	       image().Get_Image().Has_Canvas()            &&
	       image().Get_Alpha_Image().Create(dimension) &&
	       image().Get_Alpha_Image().Has_Canvas()))
	{
		image= NULL; 
		Get_Images()= -1;
		width= -1;
		return false;
	}		

	Pointer <Alpha_Pixmap> board= Board_Image::Get();

	if (! (board().Is()                        &&
	       board().Get_Image().Has_Canvas()    &&
	       board().Get_Alpha_Image().Has_Canvas()))
	{
		image= NULL; 
		Get_Images()= -1;
		width= -1; 
		return false;
	}

	GdkRectangle rect= {0, 0, board->Get_Image().Get_Size().x,
			    board->Get_Image().Get_Size().y};

	image->Get_Image()().Draw_Stretch_Canvas
		(board().Get_Image()(),
		 (GdkRectangle){0, 0, dimension.x, dimension.y},
		 rect,
		 anti_alias);

	image->Get_Alpha_Image()().Draw_Stretch_Canvas
		(board().Get_Alpha_Image()(),
		 (GdkRectangle){0, 0, dimension.x, dimension.y},
		 rect,
		 anti_alias); 

	/* save scaled image to file */ 
	Write_To_File(image_filename(), image().Get_Image());
	Write_To_File(alpha_image_filename(), image().Get_Alpha_Image());
	chmod(image_filename(), 0644);
	chmod(alpha_image_filename(), 0644);

	return true;
}

void Scaled_Board_Image::Remove()
{
	assert (width > 0);

	assert (Get_Images()[index]().width == width);
	assert (Get_Images()[index]().ref_count >= 1);

	-- Get_Images()[index]().ref_count;
	/* Keep unused sized.  */
	width= -1;
}

void Scaled_Board_Image::Draw(Canvas &canvas,
			      GdkPoint origin,
			      int pt,
			      int cc)
{
	assert (pt >= 0 && pt < pi_count);
	assert (cc == cc_w || cc == cc_b);
	
	if (! (Get_Image().Has_Canvas()   &&
	       Get_Alpha_Image().Has_Canvas()))
		return;

	canvas.Set_Function(GDK_AND_INVERT);
	canvas.Draw_Canvas(Get_Alpha_Canvas(),
			   (GdkRectangle){origin.x, origin.y,
						  width, width},
			   (GdkPoint){width * pt,
				 width * cc});
	canvas.Set_Function(GDK_OR);
	canvas.Draw_Canvas(Get_Canvas(),
			   (GdkRectangle){origin.x, origin.y,
						  width,width},
			   (GdkPoint){width * pt,
				 width * cc});
}
