#ifndef SETTINGS_HH
#define SETTINGS_HH

/* Configuration files in the home directory */

#include "filename.hh"
#include "inline.hh"
#include "dir.hh"
#include "stream_file.hh"

#include <glib/gutils.h>

/* Name of the configuration directory in the user's home directory */
#ifndef SETTINGS_DIR_NAME
#   define SETTINGS_DIR_NAME   ".babychess"
#endif

bool Settings_Make_Dir();
/* Make the configuration directory */

Inline String Settings_Filename(String name)
{
	return String(g_get_home_dir()) / SETTINGS_DIR_NAME / name;
}

template <class Type>
Inline bool Settings_Load(Type &info, String name= "")
{
	if (! Settings_Make_Dir())  return false; 
	assert (name); 
	String filename= Settings_Filename(name);
	return Read_From_File(filename(), info);
}

template <class Type>
Inline bool Settings_Save(const Type &info, String name= "")
{
	if (! Settings_Make_Dir())  return false; 
	assert (name);
	String filename= Settings_Filename(name);
	return Write_To_File(filename(), info);
}

#endif /* ! SETTINGS_HH */
